FrontlineAPI = function() {
  var errorInstance;
  this.activeContainer;
  this.wins = {};

  this.ApplicationCRUD = function(baseURL) {

    var cookieOptions = {path: '/', expires: 3650};

    this.generate = function() {
      var name = validateName();
      if(!name) return;

      if(applicationExists(name))
        return Frontline.alert('A application with same name already exists');

      var path = validatePath();
      if (!path) return;

      var url  = baseURL + '/generate';
      var data = {
        settings: $('#applicationSettingsForm').serialize(),
            uuid: Frontline.UUID(),
            name: name,
            path: path,
        preview_url: $('#previewURL').val()
      }
      $.ajax({
        type: 'POST', url: url, data: data, complete: function(xhr, txtResponse) {
          txtResponse == 'success' || Frontline.error(xhr.responseText);
        }
      });
    }

    this.load = function() {
      var name = validateName();
      if(!name) return;

      var path = validatePath();
      if (!path) return;
      
      var previewURL = $('#previewURL').val();

      var api = this;
      $.ajax({
        type: 'POST',
        url:  baseURL + '/load',
        data: {path: path},
        complete: function(xhr, txtResponse) {
          if(txtResponse == 'success') {
            if (api.addToList(JSON.stringify([name, path, previewURL])))
              location.reload();
          } else Frontline.error(xhr.responseText);
        }
      });
    }

    this.addToList = function(string) {
      var application = JSON.parse(string);
      if(applicationExists(application[0])) {
        Frontline.alert('A application with same name already exists');
        return false;
      }
      var applications = getApplications();
      if(applications && applications.push(application))
        setApplications(applications);
      return true;
    }

    this.removeFromList = function(name) {
      if(!confirm("This will remove the application from list. Continue?"))
        return false;
      
      var applications = getApplications();
      for (var i = 0; i < applications.length; i++) {
        var application = applications[i];
        if(application && application[0] == name) applications.splice(i, 1);
      }
      setApplications(applications);
      location.reload();
    }

    var applicationExists = function(name) {
      var applications = getApplications();
      for (var i = 0; i < applications.length; i++) {
        var p = applications[i];
        if(p && p[0] == name) return p;
      }
      return false;
    }

    var getApplications = function(){
      return JSON.parse($.cookie('Applications') || '[]');
    }

    var setApplications = function(applications){
      $.cookie('Applications', JSON.stringify(applications), cookieOptions);
    }

    var validateName = function() {
      var name;
      var smth = ($('#applicationName').val() || '').replace(/^\s+|\s+$/g, '');
      if(smth.length > 0)
        name = smth;
      else
        Frontline.alert('Please provide application name');
      return name;
    }

    var validatePath = function() {
      var path;
      var smth = ($('#applicationPath').val() || '').replace(/^\s+|\s+$/g, '');
      if(smth.length > 0){
        if(smth[0] == '/')
          path = smth;
        else
          Frontline.alert('Only Absolute paths accepted');
      } else
        Frontline.alert('Please provide full path to application');
      return path;
    }

  }

  this.FileCRUD = function(baseURL) {

    this.toggleEditor = function(path) {
      $.ajax({
        type: 'GET',
        url:  baseURL,
        data: {path: path},
        complete: function(xhr, txtResponse) {
          if(txtResponse == 'success') {
            $('#editorContainer').html(xhr.responseText);
          } else {
            Frontline.error(xhr.responseText);
          }
        }
      });
    }

    this.saveFile = function(path, domID) {
      $.ajax({
        type: 'PUT',
        url:  baseURL,
        data: {path: path, content: $(domID || '#editor').val()},
        complete: function(xhr, txtResponse) {
          if(txtResponse == 'success') {
            Frontline.alert('File Successfully Saved');
          } else {
            Frontline.error(xhr.responseText);
          }
        }
      });
    }
  }

  this.CRUD = function(baseURL, activeContainer) {

    this.create = function(formSelector) {
      if (isEmpty('name', $(formSelector + ' :input[name=name]').val())) return false;
      invoke('POST', $(formSelector).serialize());
    }

    this.delete = function(name, confirmation) {
      if (isEmpty('name', name)) return false;

      confirmation = confirmation || 'This action can not be undone! Continue?';
      if (confirm(confirmation))
        invoke('DELETE', {name: name});
    }

    var invoke = function(requestMethod, data) {
      Frontline.activeContainer = activeContainer;
      $.ajax({
        type: requestMethod,
        url:  baseURL + '?uuid=' + Frontline.UUID(),
        data: data,
        complete: function(xhr, txtResponse) {
          txtResponse == 'success' || Frontline.error(xhr.responseText);
        }
      });
    }
  }

  this.Migrations = function(baseURL, activeContainer) {

    this.newColumn = function(formSelector) {
      var model = $(formSelector + ' :input[name=model]').val();
      if (isEmpty('Model Name', model)) return false;

      var name = $(formSelector + ' :input[name=name]').val();
      if (isEmpty('Column Name', name)) return false;

      var type = $(formSelector + ' :input[name=type]').val();
      
      var label = $(formSelector + ' :input[name=label]').val();

      var data = {
        name: label || ['adding', model, name, 'column'].join('-'),
        column: (type ? name + ':' + type : name)
      }
      invoke('POST', data);
    }

    this.updateColumn = function(formSelector) {
      var model = $(formSelector + ' :input[name=model]').val();
      if (isEmpty('Model Name', model)) return false;

      var name = $(formSelector + ' :input[name=name]').val();
      if (isEmpty('Column Name', name)) return false;

      var type = $(formSelector + ' :input[name=type]').val();
      if (isEmpty('New Column Type', type)) return false;

      var label = $(formSelector + ' :input[name=label]').val();

      var data = {
        name: label || ['updating', model, name, 'to', type].join('-'),
        update_column: [name, type].join(':')
      }
      invoke('POST', data);
    }

    this.renameColumn = function(formSelector) {
      var model = $(formSelector + ' :input[name=model]').val();
      if (isEmpty('Model Name', model)) return false;

      var name = $(formSelector + ' :input[name=name]').val();
      if (isEmpty('Column Name', name)) return false;

      var new_name = $(formSelector + ' :input[name=new_name]').val();
      if (isEmpty('New Name', new_name)) return false;

      var label = $(formSelector + ' :input[name=label]').val();

      var data = {
        name: label || ['renaming', model, name, 'to', new_name].join('-'),
        rename_column: name + ':' + new_name
      }
      invoke('POST', data);
    }

    this.run = function(vector, formSelector) {

      var data = {migrations: [], vector: vector};

      if (formSelector) {
        $(formSelector + ' :input[name=migrations]:checked').map(function() {
          data.migrations.push($(this).val());
        });
        if(data.migrations.length == 0) {
          Frontline.alert('No migrations selected');
          return false;
        }
        if($(formSelector + ' :input[name=forceRun]').prop('checked'))
          data['force_run'] = true;
      } else {
        if(!confirm('This will run ' + vector.toUpperCase() + ' all outstanding migrations! Continue?'))
          return false;

        data['force_yes'] = true;
      }

      invoke('POST', data);
    }

    this.delete = function(name, confirmation) {
      if (isEmpty('name', name)) return false;

      confirmation = confirmation || 'This action can not be undone! Continue?';
      if (confirm(confirmation))
        invoke('DELETE', {name: name});
    }

    var invoke = function(requestMethod, data) {
      if(activeContainer)
        Frontline.activeContainer = activeContainer;
      $.ajax({
        type: requestMethod,
        url:  baseURL + '?uuid=' + Frontline.UUID(),
        data: data,
        complete: function(xhr, txtResponse) {
          txtResponse == 'success' || Frontline.error(xhr.responseText);
        }
      });
    }
  }

  this.runSpecs = function(baseURL, controller) {
    var data = {uuid: Frontline.UUID()};
    controller && (data['controller'] = controller);
    $.ajax({
      type: 'POST',
      url:  baseURL,
      data: data,
      complete: function(xhr, txtResponse) {
        txtResponse == 'success' || Frontline.error(xhr.responseText);
      }
    });
  }

  this.runCmd = function(url, confirmation) {
    if (confirmation && !confirm(confirmation)) return false;
    
    $.ajax({
      type: 'POST',
      url:  url + '?uuid=' + Frontline.UUID(),
      complete: function(xhr, txtResponse) {
        txtResponse == 'success' || Frontline.error(xhr.responseText);
      }
    });
  }

  this.Git = function(baseURL) {
    
    this.init = function() {
      if(!confirm("This will initialize or reinitialize the Git repository. Continue?")) return;
      invoke(baseURL + '/init');
    }

    this.origin = function(url) {
      var url = (url || '').replace(/^\s+|\s+$/g, '');
      if(url.length == 0) return Frontline.alert('Please provide remote URL');
      invoke(baseURL + '/origin', {url: url});
    }

    this.commit = function(message) {
      message = (message || '').replace(/^\s+|\s+$/g, '');
      if(message.length == 0) return Frontline.alert('Can not deploy without a deploy message');
      invoke(baseURL + '/commit', {message: message});
    }

    this.push = function() {
      if(!confirm("This will push last commits to remote server. Continue?")) return;
      invoke(baseURL + '/push');
    }

    var invoke = function(url, data) {
      $.ajax({
        type: 'POST',
        url:  url + '?uuid=' + Frontline.UUID(),
        data: data,
        complete: function(xhr, txtResponse) {
          txtResponse == 'success' || Frontline.error(xhr.responseText);
        }
      });
    }
  }

  this.PTYEventListener = function(streamURL) {
    var evs = new EventSource([streamURL, Frontline.UUID()].join('/'));
    evs.addEventListener('modal', function(e) {
      if (e.data == 'show') {
        $('#PTYModalHeader').html('Please wait...');
        $('#PTYModalBody').html(null);
        $('#PTYModal').modal();
      } else if (e.data == 'hide') {
        $('#PTYModalHeader').html('Completed');
        $('#PTYModal').modal('hide');
      }
      $('#lastOperationToggler').show();
    }, false);

    evs.addEventListener('content', function(e) {
      $('#PTYModalBody').append(ansi_up.ansi_to_html(e.data)).animate({
        scrollTop: $('#PTYModalBody').prop('scrollHeight')
      }, 0);
    }, false);

    evs.addEventListener('persist_application', function(e) {
      new Frontline.ApplicationCRUD().addToList(e.data);
    }, false);

    evs.addEventListener('render', function(e) {
      $(Frontline.activeContainer).html(e.data);
      $('input[name=name]').val(null);
    }, false);

    evs.addEventListener('failures', function(e) {
      $('#PTYModalHeader').html('Errored');
      $('.' + e.data).addClass('text-error');
    }, false);

    evs.addEventListener('reload', function() {
      location.reload();
    }, false);
  }

  this.UUID = function() {
    if (typeof window.eventSourceUUID == 'undefined')
      eventSourceUUID = generateUUID();
    return eventSourceUUID;
  }

  this.alert = function(msg, timeout) {
    noty({
      text:    msg,
      type:    'information',
      layout:  'topRight',
      timeout: timeout || 2000
    });
  }

  this.sticky_alert = function(msg) {
    noty({
      text:    msg,
      type:    'information',
      layout:  'topRight'
    });
  }

  this.warn = function(msg, timeout) {
    noty({
      text:    msg,
      type:    'warning',
      layout:  'topRight',
      timeout: timeout || 3000
    });
  }

  this.sticky_warn = function(msg) {
    noty({
      text:    msg,
      type:    'warning',
      layout:  'topRight'
    });
  }

  this.error = function(msg) {
    msg = msg.replace(/</gm, '&lt;').replace(/>/gm, '&gt;').
      replace(/\n|\r/gm, '<br>').replace(/\t/gm, '&nbsp;&nbsp;');
    errorInstance = noty({
      text:    '<div style="text-align: left; max-height: 280px; overflow: auto;">' + msg + '</div>',
      type:    'warning',
      layout:  'top',
      modal:   true,
      closeWith: ['button'],
      buttons: [
        {addClass: 'btn btn-primary', text: 'Close', onClick: function($noty) {
            $noty.close();
          }
        }
      ]
    });
  }

  this.dismissLastError = function() {
    errorInstance && errorInstance.close();
  }

  this.triggerSelectable = function(domSelector) {
    $(domSelector || 'select.frontline-selectable').each(function(i,e) {
      e = $(e);
      (e.attr('style') || '').match(/width/ig) || e.css('width', '10em');
      if(e.select2 == 'undefined') return false;
      e.select2({width: 'copy'});
    });
  }

  var generateUUID = function(){
    var d = new Date().getTime();
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        var r = (d + Math.random()*16)%16 | 0;
        d = Math.floor(d/16);
        return (c=='x' ? r : (r&0x7|0x8)).toString(16);
    });
    return uuid;
  }

  this.openWindow = function(domID_or_opts) {
    if (typeof domID_or_opts == "object")
      var url = domID_or_opts.url;
    else {
      var elm = $('#' + domID_or_opts);
      var url = [elm.attr('data-url'), elm.val()||''].join('/');
    }
    var win = Frontline.wins[url];
    if (!win || win.closed) {
      win = window.open(url, '_blank');
      Frontline.wins[url] = win;
    }
    win.location = url;
    win.focus();
  }

  this.routeEditor = function(url, domID) {
    $.ajax({
      type: 'GET',
      url:  url,
      complete: function(xhr, txtResponse) {
        if (txtResponse == 'success') {
          $(domID).html(xhr.responseText);
        }
        else Frontline.error(xhr.responseText);
      }
    });
  }

  this.ajaxifyTabs = function(selector) {
    $(selector).bind('show', function(e) {
      var a = $(e.target);
      $.get(a.attr('data-url'), function(r) {
        $('#' + a.attr('href').substr(1)).html(r);
      });
    });
  }

  var isEmpty = function(variable, val) {
    if (val && jQuery.trim(val).length > 0) return false;
    Frontline.warn('Wrong ' + variable + ' provided');
    return true;
  }
}

$(function(){
  $('.sticky-dropdown').click(function (e) { e.stopPropagation() });

  $('#GitCommit').keypress(function (e) {
    if (e.which == 13) {
      gitDeployer.commit($(this).val());
      e.preventDefault();
    }
  });

  $('#GitOrigin').keypress(function (e) {
    if (e.which == 13) {
      gitDeployer.origin($(this).val());
      e.preventDefault();
    }
  });
  
});
