E.new do
  mount Frontline do
    include Enginery::Helpers
    include Frontline::Helpers
    import  Frontline::Actions

    prefix = self.name.sub(/Frontline\W+/, '').downcase
    map prefix unless mapped?
    engine :Slim
    view_prefix prefix
    view_fullpath Frontline::TEMPLATES
    layout :layout
    
    # ensure any URL will be redirected to application selector
    # unless a effective application session set.
    # it should skip actions that does not require a effective application to work
    # as well as the actions that will list/set applications.
    before do
      return unless get?
      return if [:applications, :application, :streamer, :assets].include?(action_name)
      session[:application] || redirect(Frontline::Index.route(:applications))
    end

    # check whether effective application set
    # extract its path and set `@dst_root` variable
    # that will be used later to determine the effective application root.
    before do
      session[:application] && @dst_root = session[:application][1]
    end

    # running after CRUD actions of Controllers and Models controllers.
    # it is setting body to a stream that will execute a cmd and sned result to browser.
    # cmd is obtained by running the effective action.
    # before running the action, the `@name` and `@setups` variables should be set
    # as the action rely on them.
    if [Frontline::Controllers, Frontline::Models].include?(self)

      after /\A(post|delete)_(controller|route|model|migration)/ do
        @uuid   = params.delete('uuid')
        names   = params.delete('name').to_s.split
        @setups = (params.map {|k,v| [k,v]*":" if v.to_s.strip.size > 0}.compact*" ").freeze

        response.body = stream do
          failed = false
          pty_stream 'modal', 'show'

          names.each do |name|
            @name = name.freeze
            cmd = 'bundle exec enginery ' << send(action, *action_params__array)
            passed, failure_id = pty_spawn(cmd)
            unless passed
              failed = true
              pty_stream 'failures', failure_id
            end
          end

          failed || postcrud_handlers
        end
      end
    end

    # setting a error handler that will act when some exception raised.
    error 500 do |e|
      error = if e.respond_to?(:backtrace)
        e.backtrace.inject([e.message]) {|bt,l| bt << l}
      elsif e.is_a?(Array)
        e
      else
        [e.to_s]
      end
      out = rq.xhr? ? error*"\n" : render(path_to_templates('error.slim'), error: error)
      halt 500, out
    end

  end

  session :memory
  if defined?(FRONTLINE_DEBUG)
    use Rack::CommonLogger
    use Rack::ShowExceptions
  end

  settings = {server: :Thin, port: Frontline::PORT}
  defined?(FRONTLINE_PORT) && settings[:port] = FRONTLINE_PORT
  run settings
end
